% Rosenbrock: objective and its gradient
fv = @(x1,x2) 100 * ( x2  -  x1.^2).^2 + (1 -  x1 ).^2;                                 % vectorized for fast visualization
f  = @(x)     100 * (x(2) - x(1)^2)^2  + (1 - x(1))^2;                                  % non-vectorized with vector input
gradf = @(x)  [ 400*x(1)^3 + 2*x(1) - (2 + 400*x(1)*x(2))  ;  200*x(2) - 200*x(1)^2 ];  % non-vectorized with vector input
hessf = @(x)  [ -400*x(2)+1200*x(1)^2 + 2   ,   -400*x(1)  ; ... 
                -400*x(1)                   ,   200       ];

% Other test function
% fv    = @(x1,x2) (0.5*x1-0.2).^4 + (x2-0.6).^2        ;
% f     = @(x) (0.5*x(1)-0.2)^4 + (x(2)-0.6)^2          ;  % minimum at x = [0.4 ; 0.6]
% gradf = @(x) [4*0.5*(0.5*x(1)-0.2)^3  ; 2*(x(2)-0.6)] ;
% hessf = @(x) [    3*(0.5*x(1)-0.2)^2  ,  0            ; ...
%                                    0  ,  2           ];
             
% augment function with an evaluation counter (slow, but just for the statistics)
f     = augmentWithEvalCounter(f);
gradf = augmentWithEvalCounter(gradf);
hessf = augmentWithEvalCounter(hessf);

% initial point
clear x0;
x0{1} = [  1.0 ; -0.5];
x0{2} = [ -1.2 ;  1.0];
x0{3} = [-10.0 ; 12.0];

% optimization settings
tol   = 1.0d-9;   % stopping criterion || gradf || <= tol
maxit = 1000;     % maximum number of iterations 

% prepare storages
k       = 0;
x0count = length(x0);
desc    = cell(x0count,1); 
X       = cell(x0count,1); 

% do all simulations
for i = 1:x0count  % cycle through initial guesses x0
   k = k + 1;
   f('reset');               % reset evaluation counters
   gradf('reset');           % reset evaluation counters
   hessf('reset');           % reset evaluation counters
   X{k} = localnewton(f, gradf, hessf, x0{i}, tol, maxit);
   fn = f('count');          % retrieve evaluation counters
   gradfn = gradf('count');  % retrieve evaluation counters
   hessfn = hessf('count');  % retrieve evaluation counters
   itern  = size(X{k},2)-1;  % #iterations minus 1 for x0
   desc{k} = sprintf('#%2d --> %12s:  #it = %5d', k, 'local Newton', itern);
   fprintf('#%3d:  x0 = [%6.2f ; %6.2f]', k, x0{i}(1), x0{i}(2));
   fprintf('   x = [%7.4f ; %7.4f]', X{k}(1,end), X{k}(2,end));
   fprintf('   f(x) = %-8.5f', f(X{k}(:,end)));
   fprintf('   #it = %-5d   #f = %-5d   #gradf = %-5d   #hessf = %-5d', itern, fn, gradfn, hessfn);
   fprintf(' \n');
end


% display solution in individual figures
figbase = 333;
for k = 1:length(X)
   displaySol(figbase+k, fv, X{k}, desc{k})
end


